import { parser } from 'lezer-json';
import { LezerSyntax, indentNodeProp, continuedIndent, foldNodeProp } from '@codemirror/next/syntax';
import { styleTags } from '@codemirror/next/highlight';

/// Calls
/// [`JSON.parse`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/JSON/parse)
/// on the document and, if that throws an error, reports it as a
/// single diagnostic.
const jsonParseLinter = () => (view) => {
    try {
        JSON.parse(view.state.doc.toString());
    }
    catch (e) {
        if (!(e instanceof SyntaxError))
            throw e;
        const pos = getErrorPosition(e, view.state.doc);
        return [{
                from: pos,
                message: e.message,
                severity: 'error',
                to: pos
            }];
    }
    return [];
};
function getErrorPosition(error, doc) {
    let m;
    if (m = error.message.match(/at position (\d+)/))
        return Math.min(+m[1], doc.length);
    if (m = error.message.match(/at line (\d+) column (\d+)/))
        return Math.min(doc.line(+m[1]).from + (+m[2]) - 1, doc.length);
    return 0;
}

const jsonSyntax = LezerSyntax.define(parser.withProps(indentNodeProp.add({
    Object: continuedIndent({ except: /^\s*\}/ }),
    Array: continuedIndent({ except: /^\s*\]/ })
}), foldNodeProp.add({
    Object(subtree) { return { from: subtree.from + 1, to: subtree.to - 1 }; },
    Array(subtree) { return { from: subtree.from + 1, to: subtree.to - 1 }; }
}), styleTags({
    String: "string",
    Number: "number",
    "True False": "bool",
    PropertyName: "propertyName",
    null: "null",
    ",": "separator",
    "[ ]": "squareBracket",
    "{ }": "brace"
})), {
    languageData: {
        closeBrackets: { brackets: ["[", "{", '"'] },
        indentOnInput: /^\s*[\}\]]$/
    }
});
/// Returns an extension that installs the JSON syntax provider.
function json() {
    return jsonSyntax;
}

export { json, jsonParseLinter, jsonSyntax };
